#pragma once

/// Sve funkcije potrbne za pretvaranje struktura u poruke (serijalizacija) i
/// pretvaranje poruke u korisne strukture (deserijalizacija).

void serializePloca(bool ploca[3][3], char c[3]);
unsigned char boardRowToChar(bool b[3]);

void deserializePloca(char c[3], bool ploca[3][3]);
void charToBoardRow(unsigned char c, bool *row);

// Serializacije cijele ploce
// pretvorimo 2D listu u skup char-ova za slanje
// Trenutno je ovo zapis char-ova (red po red):
// Svaki char sadrzi 8 bitova informacija
/*
1. char: ploca[0][0] ploca[0][1] ploca[0][2] 0 1 1 1 1
2. char: ploca[1][0] ploca[1][1] ploca[1][2] 0 1 1 1 1
3. char: ploca[2][0] ploca[2][1] ploca[2][2] 0 1 1 1 1
*/
void serializePloca(bool ploca[3][3], char c[3]) {
  for (int i = 0; i < 3; ++i) {
    // Pripremamo plocu za slanje red po red.
    c[i] = boardRowToChar(ploca[i]);
  }
}

// Serializacija poruke bool reda ploce u char za slanje
unsigned char boardRowToChar(bool b[3]) {
  unsigned char c = 0;
  for (int i = 0; i < 3; ++i) {
    if (b[i]) {
      c |= 1 << i;
    }
  }
  // Kako dok je ploca prazna saljemo pisemo samo 0, to u poruci originalno
  // znaci da je kraj rijeci (takoznani null character.) Zato na ostale bitove
  // postaviti jedinice da nikada ta poruka ne bi bila prazna da ju s druge
  // strane primatelj ne prekine pre rano.
  c |= 1 << 4;
  c |= 1 << 5;
  c |= 1 << 6;
  c |= 1 << 7;
  return c;
}

// Deserializacija primljene poruke ploca u 2D listu za prikaz svjetljenja
// ledica.
void deserializePloca(char c[3], bool ploca[3][3]) {
  for (int i = 0; i < 3; ++i) {
    charToBoardRow(c[i], ploca[i]);
  }
}

// Deserializacija char slova u bool polje za vizualizaciju polja
void charToBoardRow(unsigned char c, bool *row) {
  for (int i = 0; i < 3; ++i) {
    row[i] = (c >> i) & 1;
  }
};