/*
Server, njemu se salju stvari originalno
Gornja ploca (trenutno oponasa zelene ledice na donjoj ploci)
A2
*/

// Ovdje su sacuvani podaci o mrezi te kako se na nju spojiti (radi sigurnosti
// je to bolje drzati u zasebnom file-u).
#include "arduino_secrets.h"
// Popis koja ledica je spojena na koji pin
#include "pinovi_ledice.h"
// Sadrzi funkcije za punjenje i citanje poruka
#include "display.h"
#include "komunikacija.h"

// Dodatni library-ji za wifi omoguciti.
#include <WiFi101.h>
#include <WiFiUdp.h>

// Sifre za spajanje na wifi
char ssid[] = SECRET_SSID;
char pass[] = SECRET_PSWD;

unsigned int localPort = 2390;
// Varijabla za spremanje nase IP adressse na wifiju.
// Neznam jel na tom wifiju mozemo postaviti staticku IP adresu?
IPAddress ip;
// Preko ove varijable cemo slati i primati nase poruke.
WiFiUDP Udp;

// Skup simbola koja cemo slati preko wifija. Sve poruke saljemo kao skup
// char-ova (svaki char ima 8 bita) i onda moramo imati dogovor sto nam koji bit
// u poruci znaci kako bi ju na serverskoj strani znali desifrirat.
char dobivena_poruka[255];

// Spremanje pomaka dobivenog od joystick-a za kretanje po poljima.
int pomak_x, pomak_y;

// Trenutni delay izvucen kao varijabla da mozemo ubrzati/usporiti kojom brzinom
// ce se glavna petalja izvoditi
int delay_ms = 100;

constexpr int kBrojBrodova = 3;

// Pamcenje stanja ploca (ovo su 2D liste za stanje ledica - upaljene/ugasene)
bool zelena_stanja[3][3]; // Mjesta svih pogadanja
bool crvena_stanja[3][3]; // Mjesta stvarnih pogodaka

bool protivnicki_brodovi[3][3];

// Kako bi prikazali na kojem smo trenutnom polju, moramo pamtiti trenutne
// indekse u listi za x i y smjer kako bi mogli blinkati odredenu ledicu.
int trenutno_polje_x = 0;
int trenutno_polje_y = 0;

int pogodeno_polje_x;
int pogodeno_polje_y;

enum Stanje {
  Inicijalizacija = 0,
  PostavljanjeBrodova = 1,
  CekanjePostavljanjeDrugoga = 2,
  IgramJa = 3,
  IgraDrugi = 4,
  Kraj = 5
} stanje;

bool igrao_sam = false;

void resetStanja() {
  for (int i = 0; i < 3; ++i) {
    for (int j = 0; j < 3; ++j) {
      zelena_stanja[i][j] = 0;
      crvena_stanja[i][j] = 0;
      protivnicki_brodovi[i][j] = 0;
    }
  }
  igrao_sam = false;
}

void setup() {
  // Otvaranje serial monitora da mozemo gledati poruke koje ispisuje arduino
  // (primarno da znamo sto u nekoj fazi radi).
  Serial.begin(9600);

  // Ispisi na display nesto, jos se ne koristi
  setupLCD();
  printajLCD("Wifi spajanje");

  stanje = PostavljanjeBrodova;

  // Postavi sve ledica na ugasene i na inicijalno stanje postavljanja.
  resetStanja();

  // Postavljanje svih pinova za ledice na OUTPUT i postavljanje stanja
  // inicijalnog na 0 (ugasene).
  for (int i = 0; i < 3; ++i) {
    for (int j = 0; j < 3; ++j) {
      pinMode(led_zeleno[i][j], OUTPUT);
      pinMode(led_crveno[i][j], OUTPUT);
    }
  }

  // Spajanje na wifi mrezu
  int status = WiFi.begin(ssid, pass);
  while (status != WL_CONNECTED) {
    Serial.println("* Can't connect, retry *");
    status = WiFi.begin(ssid, pass);
    // wait 5 seconds for connection:
    delay(5000);
  }

  // Ispis vlastite adrese i otvaranje UDP kanala za slanje poruka.
  Serial.print("You're connected to the network: ");
  ip = WiFi.localIP();
  Serial.println(ip);
  // Otvaranje kanala za slanje prema serveru
  Udp.begin(localPort);
  printajLCD("Init ...");
}

// Funkcija s kojom prolazimo po stanjima ledica sacuvanima u led_zeleno i
// led_crveno te palimo/gasimo potrebne ledice.
void paljenje_ledica() {
  for (int i = 0; i < 3; ++i) {
    for (int j = 0; j < 3; ++j) {
      if (zelena_stanja[i][j]) {
        digitalWrite(led_zeleno[i][j], HIGH);
      } else {
        digitalWrite(led_zeleno[i][j], LOW);
      }
      if (crvena_stanja[i][j]) {
        digitalWrite(led_crveno[i][j], HIGH);
      } else {
        digitalWrite(led_crveno[i][j], LOW);
      }
    }
  }
}

void posaljiNaPlocu(IPAddress adresa, char *poruka) {
  Udp.beginPacket(adresa, localPort);
  Udp.write(poruka);
  Udp.endPacket();
}

// Funkcija za blinkanje jedne specificne ledice (npr. za prikaz joystick-a)
void blinkaj(int led) {
  delay(70);
  digitalWrite(led, HIGH);
  delay(70);
  digitalWrite(led, LOW);
}

bool procitajPoruku(char *poruka) {
  int packetSize = Udp.parsePacket();
  if (packetSize) {
    // Pročitaj poruku sa buffer-a dobivenog.
    int len = Udp.read(poruka, 255);
    if (len > 0) {
      poruka[len] = 0;
    }
    return true;
  }
  return false;
}

void loop() {
  paljenje_ledica();

  // Odabir izvršavanja faze igre.
  switch (stanje) {
  case PostavljanjeBrodova:
    fazaPostavljanjaBrodova();
    break;
  case CekanjePostavljanjeDrugoga:
    fazaCekanjePostavljanjeDrugoga();
    break;
  case IgramJa:
    fazaIgramJa();
    break;
  case IgraDrugi:
    fazaIgraDrugi();
    break;
  case Kraj:
    fazaKraj();
    break;
  default:
    break;
  };

  delay(delay_ms);
}

void fazaPostavljanjaBrodova() {
  bool nesto_procitao = procitajPoruku(dobivena_poruka);
  if (nesto_procitao) {
    switch (dobivena_poruka[0]) {
    case 'S':
      stanje = (Stanje)(dobivena_poruka[1] - 1);
      printajStanje(stanje);
      break;
    default:
      break;
    }
  }
}

void fazaCekanjePostavljanjeDrugoga() {
  bool nesto_procitao = procitajPoruku(dobivena_poruka);
  if (nesto_procitao) {
    switch (dobivena_poruka[0]) {
    case 'S':
      stanje = (Stanje)(dobivena_poruka[1] - 1);
      printajStanje(stanje);
      break;
    case 'm':
      deserializePloca(&dobivena_poruka[1], protivnicki_brodovi);
      char povratna[1];
      povratna[0] = 'a';
      posaljiNaPlocu(protivnicka_donja, povratna);
      char povratna_moja[1];
      povratna_moja[0] = 'b';
      posaljiNaPlocu(moja_donja, povratna_moja);
      break;
    default:
      break;
    }
  }
}

int broj_upaljenih_ledica(bool stanja[3][3]) {
  int broj_upaljenih = 0;
  for (int i = 0; i < 3; ++i) {
    for (int j = 0; j < 3; ++j) {
      if (stanja[i][j]) {
        broj_upaljenih++;
      }
    }
  }
  return broj_upaljenih;
}

void fazaIgramJa() {
  blinkaj(led_crveno[trenutno_polje_x][trenutno_polje_y]);

  bool nesto_procitao = procitajPoruku(dobivena_poruka);
  if (nesto_procitao) {
    switch (dobivena_poruka[0]) {
    case 'S':
      stanje = (Stanje)(dobivena_poruka[1] - 1);
      printajStanje(stanje);
      break;
    case 'j':
      trenutno_polje_x = (int)(dobivena_poruka[1] - 1);
      trenutno_polje_y = (int)(dobivena_poruka[2] - 1);
      break;
    case 'p':
      pogodeno_polje_x = (int)(dobivena_poruka[1] - 1);
      pogodeno_polje_y = (int)(dobivena_poruka[2] - 1);
      zelena_stanja[pogodeno_polje_x][pogodeno_polje_y] = true;
      if (protivnicki_brodovi[pogodeno_polje_x][pogodeno_polje_y]) {
        // Pogodio sam protivnika
        crvena_stanja[pogodeno_polje_x][pogodeno_polje_y] = true;
      }
      posaljiNaPlocu(moja_donja, dobivena_poruka);
      igrao_sam = true;
      break;
    default:
      break;
    }
  }
  if (igrao_sam) {
    // Salji nazad protivnockoj doljnoj gdje si gadao
    char protivnicka_poruka[4];
    if (broj_upaljenih_ledica(crvena_stanja) == kBrojBrodova) {
      protivnicka_poruka[0] = 'l'; // Saljemo protivnicoj donjoj da je izgubio

      char poruka[1];
      poruka[0] = 'w'; // svojoj ploci da smo pobjedili
      posaljiNaPlocu(moja_donja, poruka);
    } else {
      protivnicka_poruka[0] = 'm'; // Saljemo plocu
    }
    serializePloca(zelena_stanja, &protivnicka_poruka[1]);
    posaljiNaPlocu(protivnicka_donja, protivnicka_poruka);
  }
}

void fazaIgraDrugi() {
  igrao_sam = false;

  bool nesto_procitao = procitajPoruku(dobivena_poruka);
  if (nesto_procitao) {
    switch (dobivena_poruka[0]) {
    case 'S':
      stanje = (Stanje)(dobivena_poruka[1] - 1);
      printajStanje(stanje);
      break;
    default:
      break;
    }
  }
}

void fazaKraj() {
  bool nesto_procitao = procitajPoruku(dobivena_poruka);
  if (nesto_procitao) {
    switch (dobivena_poruka[0]) {
    case 'S':
      stanje = (Stanje)(dobivena_poruka[1] - 1);
      printajStanje(stanje);
      break;
    case 'w':
      printajLCD("Pobijedili ste!");
      break;
    case 'l':
      printajLCD("Izgubili ste!");
      break;
    case 'R':
      resetStanja();
      posaljiNaPlocu(moja_donja, dobivena_poruka);
    default:
      break;
    }
    // printajLCDDrugiRed("Ponovo ?");
  }
}